'
' FILE: EFTMigrateCSVUsers
' CREATED: 01 MAR 2005 GTH
' PURPOSE: Given a path to a CSV file, and the connection information
'		   to an EFT server, create the user accounts from that CSV
'		   file on the EFT server such that their same passwords will let
'		   the users log in, and that user has the same home paths
'		   as defined in the CSV file.
'		   The format of the CSV file is as follows:
'			UserID,Password,Description,HomeDir,Access
Option Explicit


Dim oArgs, oFSO, oFile
Dim strHost, strLogin, strPassword, strCSVFile, strHomeDirIsRoot, strStripPath, strSite, strPort, bIgnorebackuperrors
Dim strFolder, strParentFolder, strFolderList, strAccess, strSkipLines
Dim iLastSlashPos, iErr, bHomeDirIsRoot, bStripPaths
Dim SFTPServer, Sites, theSite, aUser, oPerm, arPerms
Dim arUsers, arStripPaths, arLines, arElements
Dim i,j,k, iSkipLines
Dim strCSVContents, strErrorAccounts
Dim bError, strError

'==============================================================================
'
' Main
'
'==============================================================================
If (ProcessArgs=-1) then wscript.quit

' First check for CSV file existence
Set oFSO = CreateObject("Scripting.FileSystemObject")
If (not oFSO.FileExists(strCSVFile) ) Then
	WScript.Echo "The file '" & strCSVFile & "' does not exist!"
	WScript.Quit 254
End If

WScript.Echo  "Runtime Parameters:" & vbCrLf & "-------------------" & vbCrLf & _
             "strHost = " & strHost & vbCrLf & _
			 "strPort = " & strPort & vbCrLf & _
             "Login   = " & strLogin & vbCrLf & _
             "Password= " & strPassword & vbCrLf & _
			 "strSite = " & strSite & vbCrLf & _
             "CSVFile = " & strCSVFile & vbCrLf & _
             "RootHome= " & bHomeDirIsRoot & vbCrLf & _
             "STRIP:"
For j = LBound(arStripPaths) to UBound(arStripPaths)
  WScript.Echo Chr(9) & arStripPaths(j)
Next
WScript.Echo "-------------------"

Call BackupFTPCFGFile()
Call ConnectAndLogin()
arUsers = theSite.GetUsers()

' Read CSV file into memory.
Set oFile = oFSO.OpenTextFile( strCSVFile, 1 )  ' 1 = for reading
strCSVContents = oFile.ReadAll()
oFile.Close
Set oFile = nothing
Set oFSO = nothing

' We now have the contents of the CSV file; let's parse out the usernames/passwords
' We first make sure we can SPLIT each line by ensuring we only have CR, not LF.
strCSVContents = Replace( strCSVContents, vbCrLf, Chr(10) )
strCSVContents = Replace( strCSVContents, Chr(13), Chr(10) )
arLines = Split( strCSVContents, Chr(10), -1, 0 )  ' binary comparison, return all split strings

' Now iterate over all lines in the CSV, skipping the first (header) row
' For any empty row, skip it.
' For non-empty rows, split out the fields and create/modify the user as appropriate.
iSkipLines = CLng(strSkipLines)
For i = LBound(arLines)+1+iSkipLines to UBound(arLines)
	arElements = split( arLines(i), ",", -1, 0 )
	If (UBound(arElements) < 3) Then
		If ( 0 <> Len(arLines(i)) ) Then
			WScript.Echo "I cannot handle the line '" & arLines(i) & "' because it is not in required CSV format!  Skipping..."
		End If
	Else
		bError = False
		strError = ""
		If ( not FindUser( arElements(0) ) ) Then
			' User does not yet exist, so let's create him/her.
			WScript.Echo "Creating user '" & arElements(0) & "' ('" & arElements(2) & "'): pwd='" & arElements(1) & "'; homedir='" & arElements(3) & "'"
			on error resume next
			call theSite.CreateUser( arElements(0), arElements(1), 5, arElements(2) )
			if ( err.Number <> 0 ) then
				bError = True
				strError = strError & "[CreateUser-'" & err.Description & "']"
				err.clear
			end if
			on error goto 0

			' Here we sleep for 8/10 seconds because the COM admin object seems to have problems when creating
			' many users in succession very quickly.
			WScript.Sleep(800)
		End If

		' At this point we know that the user exists, so let's get a reference to the ICIClientSettings interface
		' for that user and do the desired configuration of home folders and permissions.

		WScript.Echo "Processing user '" & arElements(0) & "'..."

		'
		' STEP 1: make sure we have an existing VFS folder for the specified HOME folder; if not, create one.
		'
		strFolder = arElements(3)
		strFolder = replace( strFolder, "\", "/" )
		If ( "/" = Right(strFolder,1) ) Then
			' remove trailing slashes for now.
			strFolder = Left( strFolder, Len(strFolder)-1 )
		End If

		'
		' Additional Step:  Strip prefix paths per configuration of this script
		'   Any specified "path prefixes" will be removed here.  Stop at the first one we find.
		For j = LBound(arStripPaths) to UBound(arStripPaths)
			If ( Left(strFolder,Len(arStripPaths(j))) = arStripPaths(j) ) Then
				strFolder = Mid( strFolder, Len(arStripPaths(j))+1 )
				Exit For
			End If
		Next

		'
		' Now we find the PARENT folder of the desired home folder, and query the EFT VFS
		' for the folder listing of that parent folder.  Looking into that listing, if
		' we do not find the desired home folder, we call upon the EFT to create it.
		iLastSlashPos = InStrRev( strFolder, "/" )
		If ( iLastSlashPos > 1 ) Then
			strParentFolder = Left( strFolder, iLastSlashPos -1 )
		Else
			' If there is no slash after the initial position, then we will assume that
			' we are talking about a subfolder from the root. This is true for either non-slashed
			' entries, or properly rooted entries. For blank we will assume home folder = user name.  E.g.:
			'     "/Rio_Home"
			'     "Rio_Home"
			'     ""
			If ( 0 = Len(strFolder) ) Then
				strFolder = arElements(0)
			End If
			strParentFolder = "/"
		End If
		On Error Resume Next
		strFolderList = theSite.GetFolderList( strParentFolder )
		iErr = err.Number
		if (iErr <> 0 ) Then
			' Errors are okay here; it just means there is no physical folder yet.
			' We simply need to create it.
			'WScript.Echo "GetFolderList->Caught Error: [" & err.Description & "]"
			'bError = True
			' strError = strError & "[GetFolderList-'" & err.Description & "']"
		End If
		err.Clear

		If (iErr <> 0) or (not ( InStr( 1,strFolderList, Mid( strFolder, iLastSlashPos + 1), 1) > 0 )) Then
			' We do not have the folder already.  Create it.  NOTE that we *might* need to recursively
			' call this on parent paths if we don't have this folder's parent created yet.
			WScript.Echo Chr(9) & "Creating the physical folder '" & strFolder & "'"
			Call theSite.CreatePhysicalFolder( strFolder )
			iErr = err.Number
			if (iErr <> 0 ) Then
				WScript.Echo "CreatePhysicalFolder->Caught Error: [" & err.Description & "]"
				bError = True
				strError = strError & "[CreatePhysicalFolder-'" & err.Description & "']"
			End If
			err.Clear
		End If

		'
		' STEP 2: Set up the appropriate permissions on that folder.
		'		  Note that we assume proper INHERITED permissions; we only need to set permissions for this user.
		'
		strAccess = ""
		If (4 = UBound(arElements)) Then
			strAccess = arElements(4)
		End If
		Set oPerm = theSite.GetBlankPermission( strFolder, arElements(0) )
		iErr = err.Number
		If (iErr <> 0 ) Then
			WScript.Echo "GetFolderList->Caught Error: [" & err.Description & "]"
			bError = True
			strError = strError & "[GetFolderList-'" & err.Description & "']"
			err.Clear
		Else
			With oPerm
			    ' WO is a special case we handle first.  As long as the access is NOT "wo", then we can
			    ' add READ permissions to all other permissions types.
				If ( StrComp("WO",strAccess,1) <> 0 ) Then
					.FileDownload = 1
					.DirList = 1
					.DirShowInList = 1
					.DirShowReadOnly = 1
				End If
				' Check to see if we should be able to WRITE in here, too, by the presence
				' of a "W" in the "Access" field [as in "R/W", "WO",etc. ]
				If ( InStr(1, strAccess, "W", 1) > 0 ) Then
					.FileUpload = 1
					.FileAppend = 1
					.FileDelete = 1
					.FileRename = 1
					.DirShowHidden = 1	' I associated this with "WRITE" permissions - sounded logical.
					.DirCreate = 1
					.DirDelete = 1		' Not sure if we want this, but I will put it in.
				End If
			End With
			WScript.Echo "Setting Permission..."
			Call theSite.SetPermission( oPerm )
			iErr = err.Number
			If (iErr <> 0 ) Then
				WScript.Echo "SetPermission->Caught Error: [" & err.Description & "]"
				bError = True
				strError = strError & "[SetPermission-'" & err.Description & "']"
				err.Clear
			End If
			WScript.Echo "DONE Setting Permission."
		End If

		' STEP 3: Set up this folder as the HOME folder, mark it as ROOT.
		Set aUser = theSite.GetUserSettings( arElements(0) )
		iErr = err.Number
		If (iErr <> 0 ) Then
			WScript.Echo "GetUserSettings->Caught Error: [" & err.Description & "]"
			bError = True
			strError = strError & "[SetPermission-'" & err.Description & "']"
			err.Clear
		End If
		Call aUser.SetHomeDirString( strFolder )
		Call aUser.SetHomeDir(1)
		If ( bHomeDirIsRoot ) Then
			Call aUser.SetHomeDirIsRoot(1)
'			WScript.Echo "Setting Home Dir to Root..."
		End If
		WScript.Echo "Applying Changes..."
		Call SFTPServer.ApplyChanges()
		WScript.Echo "DONE with '" & arElements(0) & "'!" &vbCrLf
		If ( err.Number <> 0 ) Then
			WScript.Echo "Setting Use Home Folder Attributes->Caught Error: [" & err.Description & "]"
			bError = True
			strError = strError & "[HomeFolder-'" & err.Description & "']"
			err.Clear
			' Disconnect and Reconnect every 150 users created in an attempt to prevent COM/threading/timing issues.
			SFTPServer.Close
			Set theSite = nothing
			Set SFTPServer = nothing
			Call ConnectAndLogin()
		End If
		WScript.Sleep(1000)
		' Reconnect for every user account that we have CREATED on this server.
		If ( not FindUser( arElements(0) ) ) Then
			SFTPServer.Close
			Set theSite = nothing
			Set SFTPServer = nothing
			Call ConnectAndLogin()
		End If
	End If
	If ( bError ) Then
		strErrorAccounts = strErrorAccounts & arElements(0) & strError & ","
	End If
Next

WScript.Echo "Done processing CSV file."
WScript.Echo vbCrLf & "Errors occurred on the following accounts:" & vbCrLf
strErrorAccounts = Replace( strErrorAccounts, ",", vbCrLf )
WScript.Echo strErrorAccounts
WScript.Echo vbCrLf & "[Done]" & vbCrlf

'==============================================================================
'
' FindUser
'
' See if the specified user exists in our list of users for the site.
'
'==============================================================================
Function FindUser( strUsername )
Dim bRes, i
   bRes = false
   For i = LBound( arUsers ) to UBound( arUsers )
'	WScript.Echo "Does '" & strUserName & "' = '" & arUsers(i) & "'?"
      If ( strUsername = arUsers(i) ) Then
'	 WScript.Echo "YES"
         bRes = true
         Exit For
      End If
   Next
   FindUser = bRes
End Function

'==============================================================================
'
' ConnectAndLogIn
'
' This method connectst to the specified EFT server and attempts to
' log in using the supplied information.
'
'==============================================================================
Sub ConnectAndLogIn()
Dim WshShell
	' Let's check to be sure we can connect to the specified EFT Server:
	WScript.Echo "<CONNECTING TO SERVER>"
	WScript.Echo Chr(9) & "Connecting to " & strLogin & "@" & strHost & ":" & strPort & " [Site " & strSite & "]"
	Set SFTPServer = WScript.CreateObject("SFTPCOMInterface.CIServer")
	' NOTE we assume default ADMIN port of 1100 -- please chang this if you have
	' manually configured your EFT to be different.
	On Error Resume Next
	SFTPServer.Connect strHost, CLng(strPort), strLogin, strPassword

	If Err.Number <> 0 Then
	   WScript.Echo Chr(9) & "Error connecting to '" & strHost & ":" & strPort & "' -- " & err.Description & " [" & CStr(err.Number) & "]", vbInformation, "Error"
	   WScript.Echo Chr(9) & "Attempting to restart service..."
	   err.Clear
	   Set WshShell = WScript.CreateObject("WScript.Shell")
	   call WshShell.Run("net start ""globalscape eft server""", 1, true)
	   Set WshShell = nothing
	   WScript.Echo Chr(9) & "Waiting for 5 seconds for the service to initiate..."
	   WScript.Sleep 5000
		WScript.Echo Chr(9) & "Connecting to " & strLogin & "@" & strHost & ":" & strPort & " [Site " & strSite & "]"
	   SFTPServer.Connect strHost, CLng(strPort), strLogin, strPassword
	   If Err.Number <> 0 Then
	   	WScript.Echo Chr(9) & "Error connecting to '" & strHost & ":" & strPort & "' -- " & err.Description & " [" & CStr(err.Number) & "]", vbInformation, "Error"
	   	WScript.Quit 253
	   Else
	   	WScript.Echo Chr(9) & "Connected to " & strHost
           end if
	End If
	On Error GoTo 0  ' resume error trapping
	set Sites=SFTPServer.Sites
	' NOTE we are assuming we create users on the first available site on the server.
	set theSite=Sites.Item(CLng(strSite))
	WScript.Echo Chr(9) & "Connected to site '" & theSite.Name & "'" & vbCrLf
End Sub



'==============================================================================
'
'   ProcessArgs
'
'   Parse the command-line arguments.  Results are set in global variables
'   (strHost, strLogin, strPassword, strCSVFile, bHomeDirIsRoot, arStripPaths, bStripPaths ).
'
'==============================================================================
public function ProcessArgs
    Dim iCount
    Dim oArgs

    on error resume next

	' Default our arguments.  Some are required.
	strHost = ""
	strLogin = ""
	strPassword = ""
	strPort = "1100"
	strSite = "0"
	strSkipLines = "0"
	bHomeDirIsRoot = True
	bIgnorebackuperrors = False
	bStripPaths = True
	strStripPath = "d:,/home"
	arStripPaths = split( strStripPath, ",", -1, 0)

	strErrorAccounts = ""

    '
    ' Get the command-line arguments
    '
    Set oArgs = WScript.Arguments
    if oArgs.Count > 0 then
		'
		' We have command-line arguments.  Loop through them.
		'

		iCount = 0
		ProcessArgs = 0

		do while iCount < oArgs.Count

			select case oArgs.Item(iCount)
				'
				' Server name argument
				'
				case "-s"
					if( iCount + 1 >= oArgs.Count ) then
						Syntax
						ProcessArgs = -1
						exit do
					end if

					strHost = oArgs.Item(iCount+1)
					iCount = iCount + 2

				'
				' What port to connect to for EFT server. Default to 1100
				'
				case "-port"
					if( iCount + 1 >= oArgs.Count ) then
						Syntax
						ProcessArgs = -1
						exit do
					end if

					strPort = oArgs.Item(iCount+1)
					iCount = iCount + 2

				'
				' admin login name argument
				'
				case "-u"
					if( iCount + 1 >= oArgs.Count ) then
						Syntax
						ProcessArgs = -1
						exit do
					end if

					strLogin = oArgs.Item(iCount+1)
					iCount = iCount + 2

				'
				' admin password argument
				'
				case "-p"
					if( iCount + 1 >= oArgs.Count ) then
						Syntax
						ProcessArgs = -1
						exit do
					end if

					strPassword = oArgs.Item(iCount+1)
					iCount = iCount + 2

				'
				' Which site to look into.  Defaults into 1.
				'
				case "-site"
					if( iCount + 1 >= oArgs.Count ) then
						Syntax
						ProcessArgs = -1
						exit do
					end if

					strSite = oArgs.Item(iCount+1)
					iCount = iCount + 2

				'
				' CSVFile name argument
				'
				case "-f"
					if( iCount + 1 >= oArgs.Count ) then
						Syntax
						ProcessArgs = -1
						exit do
					end if

					strCSVFile = oArgs.Item(iCount+1)
					iCount = iCount + 2

				'
				' What port to connect to for EFT server. Default to 1100
				'
				case "-skip"
					if( iCount + 1 >= oArgs.Count ) then
						Syntax
						ProcessArgs = -1
						exit do
					end if

					strSkipLines = oArgs.Item(iCount+1)
					iCount = iCount + 2
				'
				' StripPaths argument - what portions of home dirs to remove from the front of a specified value
				' Defaults to "d:\" and "/home/"
				'
				case "-strip"
					if( iCount + 1 >= oArgs.Count ) then
						Syntax
						ProcessArgs = -1
						exit do
					end if

					bStripPaths = True
					strStripPath = oArgs.Item(iCount+1)
					strStipPath = Replace( strStripPath, "\", "/" )
					arStripPaths = Split( strStripPath, ",", -1, 0 )
					For j = LBound(arStripPaths) to UBound(arStripPaths)
						If ( "/" = Right( arStripPaths(j), 1 ) ) Then
							arStripPaths(j) = Left( arStripPaths(j), Len(arStripPaths(j))-1)
						End If
					Next
					iCount = iCount + 2


				'
				' HomeDir is NOT root option
				'
				case "-noroot"
					iCount = iCount + 1
					bHomeDirIsRoot  = False

				'
				' ignore errors on backing up configuration files
				'
				case "-ignorebackuperrors"
					iCount = iCount + 1
					bIgnorebackuperrors = True

				'
				' Help option
				'
				case "-?"
					Syntax
					ProcessArgs = -1
					exit do
				'
				' Invalid argument
				'
				case else
				' Display the syntax and return an error
					wscript.echo "Unknown argument: " & oArgs.Item(iCount) & vbCrLf
					Syntax
					ProcessArgs = -1
					Exit Do
			end select
		loop
	Else
		'
		' There were no command-line arguments, display the syntax
		' and return an error.
		'
		Syntax
		ProcessArgs = -1
	End if

	Set oArgs = Nothing

	If ( strHost = "" OR strLogin = "" or strPassword = "" or strCSVFile = "" ) Then
		Syntax
		ProcessArgs = -1
	End If

End function ' ProcessArgs

'==============================================================================

'==============================================================================
'
'   Syntax
'
'   Show the command-line syntax
'
'==============================================================================
public function Syntax
	wscript.echo    vbCrLf & _
		"Purpose:   Migrate users from a CSV file into an EFT server." & vbCrLf & _
		"           The CSV file should have the following format:" & vbCrLf & _
		"             UserID,Description,Password,HomeDir,Access" & vbCrLf & _
		vbCrLf & _
		"Usage:     " & wscript.scriptname & " <arguments>" & vbCrLf & _
		vbCrLf & _
		"Required Arguments:" & vbCrLf & _
		"     -s EFT Server" & vbCrLf & _
		"     -u Admin username for EFT Server" & vbCrLf & _
		"     -p Admin password" & vbCrLf & _
		"     -f Path to CSV file to process" & vbCrLf & _
		vbCrLf & _
		"Optional Arguments: " & vbCrLf & _
		"     -? This help" & vbCrLf & _
		"     -port   admin port on EFT server. Defaults to 1100" & vbCrLf & _
		"     -site   which Site ID on the server we are manipulating. Defaults to 0" & vbCrLf & _
		"     -noroot do *not* set home folders as ROOT for users. By default home folders ARE root." & vbCrLf & _
		"     -skip   number of lines in the CSV file to skip when processing file." & vbCrLf & _
		"     -strip  remove the designated prefix from paths in the CSV file; this is a comma." & vbCrLf & _
		"             separated value if more than one path. E.G.: 'd:\,/home/'" & vbCrLf & _
		"     -ignorebackuperrors  do not abort if script cannot back up configuration." & vbCrLf &  _
		vbCrLf & _
		"Example:   " & wscript.scriptname & "  -s localhost -u admin -p secret -f c:\migrate.csv -strip ""d:\,/home/"""

end function    ' Syntax

'==============================================================================
'
'   BackupFTPCFGFile
'
'   Find the location of the FTP.CFG file by looking in the SERVICES area
'   of the registry, and make backup copies of that file.
'
'==============================================================================
public Sub BackupFTPCFGFile
Dim WsShell, strImagePath, strFileName, oFSO
Dim iPos, strSourceFileName, strDestinationFileName
Dim dtNow, strDateTime, j

   strImagePath = ""
   strFileName = "FTP.CFG"


	Set WsShell = WScript.CreateObject("WScript.Shell")
	On Error Resume Next
	strImagePath = WsShell.RegRead("HKLM\SYSTEM\CurrentControlSet\Services\GlobalSCAPE EFT Server\ImagePath")
	On Error GoTo 0
	If ( Len(strImagePath) <= 0 ) Then
		WScript.Echo "Could not find EFT Service location, so could not backup the FTP.CFG file."
		If Not bIgnorebackuperrors Then
			 WScript.Quit 255
		End If
		Exit Sub
	End If

	iPos = InStrRev( strImagePath, "\" )
	strImagePath = Left( strImagePath, iPos )

	Set oFSO = WScript.CreateObject("Scripting.FileSystemObject")
	If (not oFSO.FileExists( strImagePath & "\" & strFileName ) ) Then
		WScript.Echo "Could not locate '" & strFileName & "' in the EFT Server service's folder '" & strImagePath & "'" & VbCrLf & _
					 "Therefore I will not back up the configuration file..."
		If Not bIgnorebackuperrors Then
			 WScript.Quit 255
		End If
		Exit Sub
	End If

	dtNow = Now
	strDateTime = FormatDateTime( dtNow, 2 )   ' 2 = vbShortDate
	strDateTime = strDateTime & " " & FormatDateTime( dtNow, 4 )  ' 4 = vbShortTime
	strDateTime = Replace( strDateTime, "/", "-" )
	strDateTime = Replace( strDateTime, ":", "." )
	strDateTime = Replace( strDateTime, " ", "_" )
	strSourceFileName = strImagePath & strFileName
	strDestinationFileName = strSourceFileName & "." & strDateTime & ".BAK"
	j = 0
	While ( oFSO.FileExists( strDestinationFileName ) )
		j = j + 1
		strDestinationFileName = strSourceFileName & "." & strDateTime & ".BAK." & CStr(j)
	Wend
	WScript.Echo "Backing up CONFIGURATION file to '" & strDestinationFileName & "'"
	Call oFSO.CopyFile( strSourceFileName, strDestinationFileName, false )
End Sub
